# ruff: noqa
"""add summary

Revision ID: 4552eb3f23ce
Revises: 9e53ea8bb3b0
Create Date: 2025-06-30 16:32:49.293087

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "4552eb3f23ce"
down_revision: Union[str, None] = "9e53ea8bb3b0"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("snippets", sa.Column("summary", sa.UnicodeText(), nullable=False))
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("snippets", "summary")
    # ### end Alembic commands ###
