# ruff: noqa
"""add file processing flag

Revision ID: 4073b33f9436
Revises: 4552eb3f23ce
Create Date: 2025-07-04 10:28:36.395870

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "4073b33f9436"
down_revision: Union[str, None] = "4552eb3f23ce"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "files", sa.Column("file_processing_status", sa.Integer(), nullable=False)
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("files", "file_processing_status")
    # ### end Alembic commands ###
